@@ -69,6 +69,21 @@ cairo_status_t write_func_wrapper(void* closure, const unsigned char* data, unsi
   return static_cast<cairo_status_t>((*write_func)(data, length));
 }
 
+//TODO: When we can break ABI, move the code from [read,write]_func_wrapper()
+// to c_[read,write]_func_wrapper() and remove [read,write]_func_wrapper().
+extern "C"
+{
+static cairo_status_t c_read_func_wrapper(void* closure, unsigned char* data, unsigned int length)
+{
+  return read_func_wrapper(closure, data, length);
+}
+
+static cairo_status_t c_write_func_wrapper(void* closure, const unsigned char* data, unsigned int length)
+{
+  return write_func_wrapper(closure, data, length);
+}
+} // extern "C"
+
 Surface::Surface(cairo_surface_t* cobject, bool has_reference)
 : m_cobject(nullptr)
 {
@@ -162,6 +177,24 @@ void Surface::get_device_offset(double& x_offset, double& y_offset) const
   cairo_surface_get_device_offset(const_cast<cobject*>(cobj()), &x_offset, &y_offset);
 }
 
+void Surface::set_device_scale(double x_scale, double y_scale)
+{
+  cairo_surface_set_device_scale(cobj(), x_scale, y_scale);
+  check_object_status_and_throw_exception(*this);
+}
+
+void Surface::get_device_scale(double& x_scale, double& y_scale) const
+{
+  cairo_surface_get_device_scale(const_cast<cobject*>(cobj()), &x_scale, &y_scale);
+}
+
+double Surface::get_device_scale() const
+{
+  double x_scale = 1, y_scale = 1;
+  get_device_scale(x_scale, y_scale);
+  return (x_scale + y_scale) / 2;
+}
+
 void Surface::set_fallback_resolution(double x_pixels_per_inch, double y_pixels_per_inch)
 {
   cairo_surface_set_fallback_resolution(cobj(), x_pixels_per_inch, y_pixels_per_inch);
@@ -227,9 +260,8 @@ void Surface::write_to_png_stream(const SlotWriteFunc& write_func)
     delete old_slot;
   auto slot_copy = new SlotWriteFunc(write_func);
   set_write_slot(cobj(), slot_copy);
-  auto status = cairo_surface_write_to_png_stream(cobj(),
-                                                         &write_func_wrapper,
-                                                         slot_copy /*closure*/);
+  auto status = cairo_surface_write_to_png_stream(
+    cobj(), &c_write_func_wrapper, slot_copy /*closure*/);
   check_status_and_throw_exception(status);
 }
 #endif
@@ -315,7 +347,7 @@ RefPtr<ImageSurface> ImageSurface::create_from_png_stream(const SlotReadFunc& re
 {
   auto slot_copy = new SlotReadFunc(read_func);
   auto cobject =
-    cairo_image_surface_create_from_png_stream(&read_func_wrapper, slot_copy);
+    cairo_image_surface_create_from_png_stream(&c_read_func_wrapper, slot_copy);
   check_status_and_throw_exception(cairo_surface_status(cobject));
   set_read_slot(cobject, slot_copy);
   return make_refptr_for_instance<ImageSurface>(new ImageSurface(cobject, true /* has reference */));
@@ -433,7 +465,7 @@ RefPtr<PdfSurface> PdfSurface::create_for_stream(const SlotWriteFunc& write_func
 {
   auto slot_copy = new SlotWriteFunc(write_func);
   auto cobject =
-    cairo_pdf_surface_create_for_stream(write_func_wrapper, slot_copy,
+    cairo_pdf_surface_create_for_stream(c_write_func_wrapper, slot_copy,
                                         width_in_points, height_in_points);
   check_status_and_throw_exception(cairo_surface_status(cobject));
   set_write_slot(cobject, slot_copy);
@@ -501,7 +533,7 @@ RefPtr<PsSurface> PsSurface::create_for_stream(const SlotWriteFunc& write_func,
 {
   auto slot_copy = new SlotWriteFunc(write_func);
   auto cobject =
-    cairo_ps_surface_create_for_stream(write_func_wrapper, slot_copy,
+    cairo_ps_surface_create_for_stream(c_write_func_wrapper, slot_copy,
                                        width_in_points, height_in_points);
   check_status_and_throw_exception(cairo_surface_status(cobject));
   set_write_slot(cobject, slot_copy);
@@ -603,7 +635,7 @@ RefPtr<SvgSurface> SvgSurface::create_for_stream(const SlotWriteFunc& write_func
 {
   auto slot_copy = new SlotWriteFunc(write_func);
   auto cobject =
-    cairo_svg_surface_create_for_stream(write_func_wrapper, slot_copy,
+    cairo_svg_surface_create_for_stream(c_write_func_wrapper, slot_copy,
                                         width_in_points, height_in_points);
   check_status_and_throw_exception(cairo_surface_status(cobject));
   set_write_slot(cobject, slot_copy);
